<?php

//********************************************************************
//Class: AdminTracker.php
//Author: Evan Kaiser - a.k.a. Al3c Tr3v3lyan
//Date: 7/8/06
//Purpose: For use with my script es_admintracker.txt
//         which this class is bundled with. You can
//         use this class to manually create the database
//         file that the script uses to track admins
//         on your CS:S server
//*********************************************************************

//*********************************************************************
//Instructions for use:
//
//To include the class:
//
//		include("server/path/to/AdminTracker.php");
//
//To use more servers:
//
//remove appropriate comments (//) from lines in the function start
//
//Example code to call the class for the first time:
//(This assumes there is 1 server)
//
//		$at = new AdminTracker;
//		$at->setPassword1($myPassword1);
//		$at->setAdminArray(array()); //no members yet
//		$at->start(false, true, false); //no verb, reset, no remove
//
//Example code to call the class when adding a member to the database:
//(This assumes there are 2 different servers)
//
//		$at = new AdminTracker;
//		$at->setPassword1($myPassword1);
//		$at->setPassword2($myPassword2);
//		$at->setAdminArray(array($_POST['steamid']=>$_POST['username']));
//		$at->start(false, false, false); //no verb, no reset, no remove
//
//Example code to call the class and edit the steamid of a specific member:
//(This assumes there are 3 different servers, and you want verbose info)
//
//		$at = new AdminTracker;
//		$at->setPassword1($myPassword1);
//		$at->setPassword2($myPassword2);
//		$at->setPassword3($myPassword3);
//		$at->setAdminToRemove(array($_POST['steamid']));
//		$at->setAdminArray(array($_POST['newsteam']=>$_POST['username']));
//		$at->start(true, false, true); //verb, no reset, remove
//
//Example code to call the class and remove a member from the database:
//(This assumes that there is 1 server)
//
//		$at = new AdminTracker;
//		$at->setPassword1($myPassword1);
//		$at->setAdminToRemove(array($_POST['username']));
//		$at->setAdminArray(array());
//		$at->start(false, false, true); //no verb, no reset, remove
//
//*********************************************************************

class AdminTracker {

//*********
//FIELDS
//*********

//ftp address to access server
	var $ftpserver1 = "your_ftp_server"; 
	var $ftpserver2 = "your_ftp_server"; 
	var $ftpserver3 = "your_ftp_server"; 

//path to cfg directory on remote server. usually should be addons/eventscripts/admintracker/ unless your ftp does not go to your cstrike directory
	var $dir1 = "addons/eventscripts/admintracker/";
	var $dir2 = "addons/eventscripts/admintracker/";
	var $dir3 = "addons/eventscripts/admintracker/";

//ftp username
	var $user1 = "ftp_username";
	var $user2 = "ftp_username";
	var $user3 = "ftp_username";

//ftp password
	var $password1;
	var $password2;
	var $password3;

//name of database (don't change this variable)
	var $db = "at_admin";

//name of file to modify (don't change this variable)
	var $db_file = "es_at_admin_db.txt";

//path on the webserver where you would like to store the temporary file
	var $localpath = "/home/www/temp/";

//array that has the names and steamids of admins to add to the database
	var $admin_array;

//has the steamid of an admin to remove from the database
	var $admin_remove;

//buffer that will hold previous db file and then will hold the admins 
//you are adding from admin_array
	var $buffer = "";

//boolean flag to say whether download is complete
	var $downloadComplete = false;

//boolean flag to say whether upload is complete
	var $uploadComplete = false;


//*****************
//PUBLIC METHODS
//*****************

	//FUNCTION: setPassword1
	//Pre: password exists somewhere outside of this class
	//Post: set this->password1 to password

	function setPassword1($password){
		$this->password1 = $password;
	}


	//FUNCTION: setPassword2
	//Pre: password exists somewhere outside of this class
	//Post: set this->password2 to password

	function setPassword2($password){
		$this->password2 = $password;
	}


	//FUNCTION: setPassword3
	//Pre: password exists somewhere outside of this class
	//Post: set this->password3 to password

	function setPassword3($password){
		$this->password3 = $password;
	}

	
	//FUNCTION: setAdminArray
	//Pre: inArray exists of steamid keys and name values
	//     i.e. $array("STEAM:0:0:1234567"=>"[CLAN] - Pwner",
	//		   "STEAM:0:1:7654321"=>"[CLAN] - Noob")
	//Post: set this->admin_array to inArray

	function setAdminArray($inArray){
		$this->admin_array = $inArray;
	}
	

	//FUNCTION: setAdminToRemove
	//Pre: inSteam exists and has a value
	//Post: set this->admin_remove to inSteam

	function setAdminToRemove($inSteam){
		$this->admin_remove = $inSteam;
	}

	
	//FUNCTION: start
	//Pre: starts the ftp process
	//     verbose defines whether information should be printed to page
	//
	//     reset is a boolean to define whether to make a fresh file
	//
	//     remove is a boolean to define whether to remove a specific
	//     user or users from the list (set in setAdminRemoveArray)
	//
	//Post: new database file is updated

	function start($verbose, $reset, $remove){
		if(!$reset){
			$this->downloadFile($this->ftpserver1, 21, $this->dir1, $this->user1, $this->password1, $verbose);
			//$this->downloadFile($this->ftpserver2, 21, $this->dir2, $this->user2, $this->password2, $verbose);
			//$this->downloadFile($this->ftpserver3, 21, $this->dir3, $this->user3, $this->password3, $verbose);

			$this->readIntoBuffer($verbose);
		}
		else {
			$this->resetBuffer();
		}

		$this->addAdminsToBuffer();

		if($remove){
			$this->removeAdminsFromBuffer();
		}

		$this->writeBufferToFile($verbose);

		$this->uploadFile($this->ftpserver1, 21, $this->dir1, $this->user1, $this->password1, $verbose);
		//$this->uploadFile($this->ftpserver2, 21, $this->dir2, $this->user2, $this->password2, $verbose);
		//$this->uploadFile($this->ftpserver3, 21, $this->dir3, $this->user3, $this->password3, $verbose);

		$this->deleteFile($verbose);

		if($verbose){
			echo "<br>Database edit complete!<br>";
		}
	}


//*****************
//PRIVATE METHODS
//*****************

	//FUNCTION: downloadFile
	//Pre: server, port, user, password, and directory must exist 
	//     as the variables that will specify where to download the
	//     file from using ftp protocol:
	//     ftp://$user:$password@$server:$port/$dirctory
	//     verbose is a boolean that specifies whether to announce on
	//     the website what is happening during the process
	//Post: previous database file will be downloaded from ftp server
	function downloadFile($server, $port, $directory, $user, $password, $verbose){
		if(!$ftpStream = ftp_connect($server, $port)){
			die("<br>FTP CONNECT ERROR");
		}
		ftp_set_option($ftpStream, FTP_TIMEOUT_SEC, 120);
		if(ftp_login($ftpStream, $user, $password)){
			if($verbose){
				echo "<br><br>Logged into ".$server;
			}

			if(ftp_pasv($ftpStream, TRUE)){
				if($verbose){
					echo "<br>Passive mode entered";
				}
			}
			else{
				die("<br>FTP PASSIVE MODE ERROR");
			}

			if(ftp_chdir($ftpStream, $directory)){
				if($verbose){
					echo "<br>CWD changed to ".ftp_pwd($ftpStream);
				}
			}
			else{
				die("<br>FTP CHDIR ERROR");
			}

			if(ftp_get($ftpStream, $this->localpath.$this->db_file, $this->db_file, FTP_BINARY)){
				if($verbose){
					echo "<br>".$this->db_file." downloaded successfully!";
				}
			}
			else{
				die("<br>FTP GET ERROR");
			}
		}
		else{
			die("<br>FTP LOGIN ERROR<br>");
		}
		$this->downloadComplete = true;
		ftp_quit($ftpStream);
	}


	//FUNCTION: uploadFile
	//Pre: server, port, user, password, and directory must exist 
	//     as the variables that will specify where to upload the
	//     file using ftp protocol:
	//     ftp://$user:$password@$server:$port/$dirctory
	//     verbose is a boolean that specifies whether to announce on
	//     the website what is happening during the process
	//Post: new database file will be uploaded to ftp server
	function uploadFile($server, $port, $directory, $user, $password, $verbose){
		
		if(!$ftpStream = ftp_connect($server, $port)){
			die("<br>FTP CONNECT ERROR");
		}
		ftp_set_option($ftpStream, FTP_TIMEOUT_SEC, 120);
		if(ftp_login($ftpStream, $user, $password)){
			if($verbose){
				echo "<br><br>Logged into ".$server;
			}

			if(ftp_pasv($ftpStream, TRUE)){
				if($verbose){
					echo "<br>Passive mode entered";
				}
			}
			else{
				die("<br>FTP PASSIVE MODE ERROR");
			}

			if(ftp_chdir($ftpStream, $directory)){
				if($verbose){
					echo "<br>CWD changed to ".ftp_pwd($ftpStream);
				}
			}
			else{
				die("<br>FTP CHDIR ERROR");
			}

			if(@ftp_delete($ftpStream, $this->db_file)){
				if($verbose){
					echo "<br>Previous ".$this->db_file." deleted successfully!";
				}
			}
			else{
				if($verbose){
					echo "<br>No remote ".$this->db_file." file to delete.";
				}
			}

			if(ftp_put($ftpStream, $this->db_file, $this->localpath.$this->db_file, FTP_BINARY)){
				if($verbose){
					echo "<br>".$this->db_file." uploaded successfully!";
				}
			}
			else{
				die("<br>FTP PUT ERROR");
			}
		}
		else{
			die("<br>FTP LOGIN ERROR<br>");
		}
		$this->uploadComplete = true;
		ftp_quit($ftpStream);
	}

	
	//FUNCTION: readIntoBuffer
	//Pre: downloadFile has been called and completed successfully
	//     verbose is a boolean value that tells whether to display information on site
	//Post: will read the contents of local file into this->buffer
	function readIntoBuffer($verbose){
		while(!$this->downloadComplete){
			if($verbose){
				echo "<br>LOGIC ERROR - Download not complete yet<br>";
			}
		}
		$this->buffer = "";
		$file = fopen($this->localpath.$this->db_file, "r");
		if($file) {
			while(!feof($file)){
				$this->buffer .= fgets($file, 4096);
			}
		}
		else {
			die("<br>LOCAL FGETS ERROR");
		}
	}

	
	//FUNCTION: writeBufferToFile
	//Pre: verbose is a boolean value that tells whether to display information on site
	//Post: will write this->buffer into the local temp file
	function writeBufferToFile($verbose){
		$file = fopen($this->localpath.$this->db_file, "w");
		if(!fputs($file, $this->buffer)){
			die("<br>LOCAL FPUTS ERROR");
		}

	}


	//FUNCTION: deleteFile
	//Pre: local temp file exists and file upload was completed sucessfully
	//Post: local temp file is deleted
	function deleteFile($verbose){
		while(!$this->uploadComplete){
			if($verbose){
				echo "<br>LOGIC ERROR - Upload not complete yet<br>";
			}
		}
		$file = $this->localpath.$this->db_file;
		if(unlink($file)){
			if($verbose){
				echo "<br>Previous ".$this->db_file." deleted successfully!";
			}
		}
		else{
			if($verbose){
				echo "<br>No local ".$this->db_file." file to delete.";
			}
		}
	}


	//FUNCTION: resetBuffer
	//Pre: this->buffer contains previous database from temp file
	//Post: delete everything in it and start over
	function resetBuffer(){
		$newbuf = "\"".$this->db."\"".chr(13).chr(10);
		$newbuf .= "{".chr(13).chr(10);
		$newbuf .= "}".chr(13).chr(10);
		$this->buffer = $newbuf;
	}


	//FUNCTION: addAdminsToBuffer
	//Pre: this->buffer contains previous database from temp file
	//Post: adds keys for each steamid and the values such as name required for they key
	function addAdminsToBuffer(){
		$i = strlen($this->buffer);
		while(($i >= 0) && ($this->buffer[$i] != "}")){
			$i--;
		}
		$newbuf = substr($this->buffer, 0, $i);

		do {
			$newbuf .= "    \"".trim(key($this->admin_array))."\"".chr(13).chr(10);
			$newbuf .= "    {".chr(13).chr(10);
			$newbuf .= "        \"name\" \"".current($this->admin_array)."\"".chr(13).chr(10);
			$newbuf .= "        \"totaltime\" \"0-0-0\"".chr(13).chr(10);
			$newbuf .= "        \"previoustotal\" \"0-0-0\"".chr(13).chr(10);
			$newbuf .= "    }".chr(13).chr(10);
		} while(next($this->admin_array));
		$newbuf .= "}".chr(13).chr(10);

		$this->buffer = $newbuf;
	}


	//FUNCTION: removeAdminsFromBuffer
	//Pre: this->buffer contains previous database from temp file
	//Post: removes the key that is for member with steamid admin_remove
	function removeAdminsFromBuffer(){
		$endbuf = strstr($this->buffer, $this->admin_remove);
		$endbuf = strstr($endbuf, "}");
		$endbuf = substr($endbuf, 3, strlen($endbuf)-3);
		$beginbuf = substr($this->buffer, 0, strpos($this->buffer, $this->admin_remove) - 5);
		$this->buffer = $beginbuf.$endbuf;
	}
}
?>